"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBodyAssets = exports.getHeadAssets = exports.getAbsoluteAssetPath = exports.getAssets = exports.getAssetLocations = exports.updateAssetLocations = exports.setDefaultAssets = exports.getAssetsFile = void 0;
var tslib_1 = require("tslib");
var fs_1 = tslib_1.__importDefault(require("fs"));
var path_1 = tslib_1.__importDefault(require("path"));
var assets;
var assetsLoaded = false;
function getAssetsFile(base) {
    return path_1.default.resolve(base, 'assets.json');
}
exports.getAssetsFile = getAssetsFile;
function ensureAssets(runtimeConfig) {
    if (assetsLoaded) {
        return;
    }
    var assetsFile = getAssetsFile(runtimeConfig.BASE);
    try {
        var assetsFileContents = fs_1.default.readFileSync(assetsFile);
        assets = JSON.parse(assetsFileContents.toString());
        assetsLoaded = true;
    }
    catch (e) {
        throw new Error("Cannot read assets file from ".concat(assetsFile));
    }
}
function setDefaultAssets(buildConfig) {
    updateAssetLocations({
        main: {
            css: buildConfig.PUBLIC_PATH + buildConfig.ASSETS_PATH_PREFIX + 'css/main.css',
            js: buildConfig.PUBLIC_PATH + buildConfig.ASSETS_PATH_PREFIX + 'js/main.js',
        },
    });
}
exports.setDefaultAssets = setDefaultAssets;
function updateAssetLocations(newAssets) {
    assets = newAssets;
    assetsLoaded = true;
}
exports.updateAssetLocations = updateAssetLocations;
function getAssetLocations(runtimeConfig) {
    ensureAssets(runtimeConfig);
    return assets;
}
exports.getAssetLocations = getAssetLocations;
function getAssets(runtimeConfig) {
    ensureAssets(runtimeConfig);
    return assets;
}
exports.getAssets = getAssets;
function getAbsoluteAssetPath(runtimeConfig, asset) {
    var staticPath = runtimeConfig.PUBLIC_PATH + runtimeConfig.ASSETS_PATH_PREFIX;
    var relativeAsset = asset.slice(staticPath.length);
    if (relativeAsset[0] === '/') {
        relativeAsset = relativeAsset.slice(1);
    }
    return path_1.default.resolve(runtimeConfig.ASSETS_PATH, relativeAsset);
}
exports.getAbsoluteAssetPath = getAbsoluteAssetPath;
function getHeadAssets(buildAssets) {
    return Object.keys(buildAssets)
        .filter(function (chunkName) {
        var chunk = buildAssets[chunkName];
        return chunk.css;
    })
        .map(function (chunkName) { return ({
        tag: "<link id=\"css-main\" type=\"text/css\" rel=\"stylesheet\" href=\"".concat(buildAssets[chunkName].css, "\">"),
    }); });
}
exports.getHeadAssets = getHeadAssets;
function getBodyAssets(buildAssets) {
    var chunkNames = Object.keys(buildAssets).filter(function (chunkName) {
        if (chunkName === 'main' || chunkName === 'manifest') {
            return false;
        }
        var chunk = buildAssets[chunkName];
        return !!chunk.js;
    });
    return tslib_1.__spreadArray(tslib_1.__spreadArray(['manifest'], chunkNames, true), ['main'], false).filter(function (chunkName) { return buildAssets[chunkName]; })
        .reduce(function (acc, chunkName) {
        var jsAsset = buildAssets[chunkName].js;
        if (Array.isArray(jsAsset)) {
            acc.push.apply(acc, jsAsset.map(function (file) { return ({
                tag: "<script src=\"".concat(file, "\"></script>"),
            }); }));
        }
        else {
            acc.push({
                tag: "<script src=\"".concat(buildAssets[chunkName].js, "\"></script>"),
            });
        }
        return acc;
    }, []);
}
exports.getBodyAssets = getBodyAssets;
