"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRuntimeConfig = exports.getRuntimeConfigFromBuildConfig = exports.watchtowerConfigFilename = exports.getBuildConfig = void 0;
var tslib_1 = require("tslib");
var path_1 = tslib_1.__importDefault(require("path"));
var __1 = require("..");
function getDefaultConfig(root) {
    return {
        ASSETS_PATH_PREFIX: 'static/',
        BASE: root,
        CLIENT_ENTRY: path_1.default.resolve(root, 'client', 'index.tsx'),
        CLIENT_POLYFILLS: path_1.default.resolve(root, 'client', 'polyfills.ts'),
        DEV_SERVER_PORT: 3000,
        HAS_SERVER: true,
        OUTPUT: path_1.default.resolve(root, 'dist'),
        PUBLIC_PATH: '/',
        SERVER_BUNDLE_ALL: false,
        SERVER_ENTRY: path_1.default.resolve(root, 'server', 'start.ts'),
        SERVER_INCLUDE_IN_BUNDLE: ['project-watchtower'],
        SERVER_PUBLIC_DIR: path_1.default.resolve(root, 'public'),
        STATIC_RESOURCE_NAMES: false,
        STATS_ENV: {},
        STATS_PAGES: { home: '/' },
    };
}
function getCustomConfig(log, root) {
    return (0, __1.getCustomConfigFile)(log, root, 'config/config', {});
}
function getBuildConfig(log, root) {
    return tslib_1.__assign(tslib_1.__assign({}, getDefaultConfig(root)), getCustomConfig(log, root));
}
exports.getBuildConfig = getBuildConfig;
exports.watchtowerConfigFilename = 'watchtower.config';
function getRuntimeConfigFromBuildConfig(buildConfig) {
    return {
        ASSETS_PATH: path_1.default.join(buildConfig.OUTPUT, buildConfig.ASSETS_PATH_PREFIX),
        ASSETS_PATH_PREFIX: buildConfig.ASSETS_PATH_PREFIX,
        BASE: buildConfig.OUTPUT,
        PUBLIC_PATH: buildConfig.PUBLIC_PATH,
        SERVER_PUBLIC_DIR: buildConfig.SERVER_PUBLIC_DIR,
    };
}
exports.getRuntimeConfigFromBuildConfig = getRuntimeConfigFromBuildConfig;
function getRuntimeConfig(log) {
    var projectDirEnv = (0, __1.projectDir)();
    if (projectDirEnv) {
        log.debug({ projectDir: projectDirEnv }, 'Project dir specified, using build config to build runtime config');
        var config = getBuildConfig(log, projectDirEnv);
        return getRuntimeConfigFromBuildConfig(config);
    }
    return JSON.parse((0, __1.readFileSync)(log, exports.watchtowerConfigFilename));
}
exports.getRuntimeConfig = getRuntimeConfig;
