"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDefaultHtmlMiddleware = exports.expressNoop = void 0;
var tslib_1 = require("tslib");
var fs_1 = tslib_1.__importDefault(require("fs"));
var path_1 = tslib_1.__importDefault(require("path"));
var assets_1 = require("../assets");
var render_html_1 = require("../ssr/helpers/render-html");
var promise_tracker_1 = require("../utils/promise-tracker");
var isProduction = process.env.NODE_ENV === 'production';
var expressNoop = function (_res, _req, next) { return next(); };
exports.expressNoop = expressNoop;
function getDefaultHtmlMiddleware(log, runtimeConfig) {
    if (isProduction) {
        var indexPath_1 = path_1.default.resolve(runtimeConfig.BASE, 'index.html');
        var indexFileMissing_1 = !fs_1.default.existsSync(indexPath_1);
        var loggedMissingIndexFileError_1 = false;
        var productionMiddleware = function (_req, res) {
            if (indexFileMissing_1) {
                if (!loggedMissingIndexFileError_1) {
                    log.error('Cannot find index.html file to serve, default watchtower middleware will return 404');
                    loggedMissingIndexFileError_1 = true;
                }
                return res.status(404);
            }
            return res.status(200).sendFile(indexPath_1);
        };
        return productionMiddleware;
    }
    var indexContent;
    if (runtimeConfig.SERVER_PUBLIC_DIR) {
        var indexPath = path_1.default.resolve(runtimeConfig.SERVER_PUBLIC_DIR, 'index.html');
        if (fs_1.default.existsSync(indexPath)) {
            indexContent = fs_1.default.readFileSync(indexPath, 'utf8');
        }
    }
    var middleware = function (req, res) {
        var buildAssets = (0, assets_1.getAssets)(runtimeConfig);
        if (indexContent) {
            var withHeadTags = indexContent.replace('</head>', "".concat((0, assets_1.getHeadAssets)(buildAssets)
                .map(function (headAsset) { return headAsset.tag; })
                .join('\n'), "\n    </head>"));
            var withBodyTags = withHeadTags.replace('</body>', "".concat((0, assets_1.getBodyAssets)(buildAssets)
                .map(function (bodyAsset) { return bodyAsset.tag; })
                .join('\n'), "\n    </body>"));
            return res.status(200).contentType('text/html').send(withBodyTags);
        }
        var indexWithAssets = (0, render_html_1.renderHtml)({
            context: {
                promiseTracker: new promise_tracker_1.PromiseTracker(),
                ssrRequestProps: {},
            },
            head: undefined,
            pageTags: {
                body: (0, assets_1.getBodyAssets)(buildAssets),
                preBody: [],
                head: (0, assets_1.getHeadAssets)(buildAssets),
            },
            renderResult: '',
            req: req,
        });
        return res.status(200).contentType('text/html').send(indexWithAssets);
    };
    return middleware;
}
exports.getDefaultHtmlMiddleware = getDefaultHtmlMiddleware;
