"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createServer = exports.getPort = void 0;
var tslib_1 = require("tslib");
var express_1 = tslib_1.__importDefault(require("express"));
var assets_1 = require("./assets");
var default_html_middleware_1 = require("./middleware/default-html-middleware");
var ensure_request_log_middleware_1 = require("./middleware/ensure-request-log-middleware");
var config_1 = require("./config/config");
var environment_1 = require("./environment");
var network_1 = require("./utils/network");
var isProduction = process.env.NODE_ENV === 'production';
var getPort = function (fallbackPort) {
    var port = parseInt(process.env.PORT || '', 10) || fallbackPort;
    if (!port) {
        throw new Error('No port specified, set one through the PORT environmental variable');
    }
    return port;
};
exports.getPort = getPort;
function createServer(options) {
    var config = (0, config_1.getRuntimeConfig)(options.log);
    var earlyMiddlewareHook = options.earlyMiddlewareHook, middlewareHook = options.middlewareHook, callback = options.callback, _a = options.startListening, startListening = _a === void 0 ? true : _a;
    var app = (0, express_1.default)();
    app.log = options.log;
    app.disable('x-powered-by');
    if ((0, environment_1.isWatchtowerServer)()) {
        var buildConfig = (0, config_1.getBuildConfig)(options.log, (0, environment_1.projectDir)() || process.cwd());
        (0, assets_1.setDefaultAssets)(buildConfig);
    }
    if (process.env.NODE_ENV !== 'production' && (0, environment_1.isWatchMode)()) {
        var buildConfig = (0, config_1.getBuildConfig)(options.log, (0, environment_1.projectDir)() || process.cwd());
        (0, assets_1.setDefaultAssets)(buildConfig);
        var getHotReloadMiddleware = require('@project-watchtower/cli').getHotReloadMiddleware;
        app.use(getHotReloadMiddleware(options.log, buildConfig));
    }
    app.use((0, ensure_request_log_middleware_1.createEnsureRequestLogMiddleware)(options.log));
    if (earlyMiddlewareHook) {
        earlyMiddlewareHook(app);
    }
    var assetsPathPrefixWithLeadingSlash = config.ASSETS_PATH_PREFIX[0] === '/'
        ? config.ASSETS_PATH_PREFIX
        : "/".concat(config.ASSETS_PATH_PREFIX);
    app.use(assetsPathPrefixWithLeadingSlash, express_1.default.static(config.ASSETS_PATH, {
        index: false,
    }));
    if (config.SERVER_PUBLIC_DIR) {
        app.use(express_1.default.static(config.SERVER_PUBLIC_DIR, {
            index: false,
        }));
    }
    if (middlewareHook) {
        middlewareHook(app);
    }
    app.get('*', (0, default_html_middleware_1.getDefaultHtmlMiddleware)(options.log, config));
    if (!startListening) {
        return app;
    }
    var listen = function (usePort) {
        var server = app.listen(usePort, function () {
            options.log.info("Server listening on port ".concat(usePort));
            if (process.env.NODE_ENV !== 'production' && (0, environment_1.isWatchMode)()) {
                var openBrowser = require('@project-watchtower/cli').openBrowser;
                openBrowser(options.log, usePort);
            }
            if (callback) {
                callback();
            }
        });
        app.set('server', server);
    };
    var port = (0, exports.getPort)();
    if (isProduction) {
        listen(port);
    }
    else {
        (0, network_1.findFreePort)(port).then(function (usePort) { return listen(usePort); });
    }
    return app;
}
exports.createServer = createServer;
