"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderPageContents = void 0;
var tslib_1 = require("tslib");
var runtime_1 = require("@project-watchtower/runtime");
var recursive_task_resolver_1 = require("./helpers/recursive-task-resolver");
var render_app_to_string_1 = require("./render-app-to-string");
var router_context_handler_1 = require("./router-context-handler");
var ServerRenderResults = tslib_1.__importStar(require("./server-render-results"));
var runtime_2 = require("@project-watchtower/runtime");
function renderErrorRoute(ssrRequestProps, options, currentRenderLocation, promiseTracker, err, startTime) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var errorRenderOptions, errorRender;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    errorRenderOptions = err instanceof runtime_1.Status404Error
                        ? { location: options.pageNotFoundLocation, status: 404 }
                        : { location: options.errorLocation, status: 500 };
                    if (currentRenderLocation === errorRenderOptions.location) {
                        throw err;
                    }
                    options.resetRequest(errorRenderOptions.location);
                    currentRenderLocation = errorRenderOptions.location;
                    return [4, renderWithErrorPageFallback(ssrRequestProps, options, errorRenderOptions.location, promiseTracker, startTime)];
                case 1:
                    errorRender = _a.sent();
                    if (errorRender.type === ServerRenderResults.ServerRenderResultType.Success) {
                        errorRender.statusCode = errorRenderOptions.status;
                    }
                    return [2, errorRender];
            }
        });
    });
}
function performRender(ssrRequestProps, options, renderLocation, promiseTracker, startTime) {
    var storeStateAtRenderTime = tslib_1.__assign({}, ssrRequestProps);
    try {
        var renderResult = (0, render_app_to_string_1.renderApp)(renderLocation, options.renderFn, options.log, options.appRender, promiseTracker);
        var result = (0, router_context_handler_1.routerContextHandler)(renderResult, startTime, storeStateAtRenderTime, renderLocation);
        return result;
    }
    finally {
        if (options.events && options.events.renderPerformed) {
            try {
                options.events.renderPerformed(promiseTracker);
            }
            catch (err) {
                options.log.error({ err: err }, 'renderPerformed event failed');
            }
        }
    }
}
function renderWithErrorPageFallback(ssrRequestProps, options, renderLocation, promiseTracker, startTime) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var initialRenderResult, dataResolved, dataLoadErr_1;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    _a.trys.push([0, 2, , 4]);
                    initialRenderResult = performRender(ssrRequestProps, options, renderLocation, promiseTracker, startTime);
                    if (initialRenderResult.type !== ServerRenderResults.ServerRenderResultType.Success) {
                        return [2, initialRenderResult];
                    }
                    if (options.events && options.events.beginWaitingForTasks) {
                        try {
                            options.events.beginWaitingForTasks((0, runtime_2.formatElapsed)(process.hrtime(startTime)));
                        }
                        catch (err) {
                            options.log.error({ err: err, location: location }, 'beginWaitingForTasks threw, continuing');
                        }
                    }
                    return [4, (0, recursive_task_resolver_1.recursiveTaskResolver)(options.log, promiseTracker, function () {
                            return performRender(ssrRequestProps, options, renderLocation, promiseTracker, startTime);
                        }, renderLocation, initialRenderResult, 10, options.ssrTimeoutMs)];
                case 1:
                    dataResolved = _a.sent();
                    return [2, dataResolved];
                case 2:
                    dataLoadErr_1 = _a.sent();
                    if (dataLoadErr_1 instanceof runtime_1.Information) {
                        options.log.info({ err: dataLoadErr_1 }, 'Data load warning, rendering error location');
                    }
                    else if (dataLoadErr_1 instanceof runtime_1.Warning) {
                        options.log.warn({ err: dataLoadErr_1 }, 'Data load warning, rendering error location');
                    }
                    else {
                        options.log.error({ err: dataLoadErr_1 }, 'Data load failed, rendering error location');
                    }
                    return [4, renderErrorRoute(ssrRequestProps, options, renderLocation, promiseTracker, dataLoadErr_1, startTime)];
                case 3: return [2, _a.sent()];
                case 4: return [2];
            }
        });
    });
}
function renderPageContents(ssrRequestProps, options, location, promiseTracker) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var startTime, err_1, failure;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    startTime = process.hrtime();
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4, renderWithErrorPageFallback(ssrRequestProps, options, location, promiseTracker, startTime)];
                case 2: return [2, _a.sent()];
                case 3:
                    err_1 = _a.sent();
                    failure = {
                        elapsed: (0, runtime_2.formatElapsed)(process.hrtime(startTime)),
                        errorMessage: 'Failed to do render',
                        head: undefined,
                        type: ServerRenderResults.ServerRenderResultType.Failure,
                    };
                    options.log.error({ err: err_1 }, 'Failed to render');
                    return [2, failure];
                case 4: return [2];
            }
        });
    });
}
exports.renderPageContents = renderPageContents;
