"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var typescript_log_1 = require("typescript-log");
var recursive_task_resolver_1 = require("./recursive-task-resolver");
var promise_tracker_1 = require("../../utils/promise-tracker");
var testLogger = (0, typescript_log_1.consoleLogger)();
describe('Recursive Task Resolver', function () {
    it('Should abort after 5 tries', function (done) {
        var promiseTracker = new promise_tracker_1.PromiseTracker();
        var count = 0;
        var render = function () {
            if (count++ > 5) {
                done(new Error('Too many render calls!'));
            }
            promiseTracker.track(new Promise(function (resolve) { return setTimeout(resolve, 10); }));
            return '';
        };
        render();
        (0, recursive_task_resolver_1.recursiveTaskResolver)(testLogger, promiseTracker, render, '/', '', 5, 500)
            .then(function () { return done(new Error('Expected too many recurses error')); }, function (err) {
            return expect(err.message).toEqual('Abort waiting for loading all data after 5 recursive waits');
        })
            .then(function () { return done(); })
            .catch(done);
    });
    it('Resolves when nested promise resolves', function (done) {
        var promiseTracker = new promise_tracker_1.PromiseTracker();
        var count = 0;
        var resolvedSecondPromise = false;
        var render = function () {
            if (count === 0) {
                promiseTracker.track(new Promise(function (resolve) { return setTimeout(resolve, 10); }));
            }
            else if (count === 1) {
                var secondPromise = new Promise(function (resolve) { return setTimeout(resolve, 10); }).then(function () {
                    resolvedSecondPromise = true;
                });
                promiseTracker.track(secondPromise);
            }
            count++;
            return '';
        };
        render();
        (0, recursive_task_resolver_1.recursiveTaskResolver)(testLogger, promiseTracker, render, '/', '', 5, 100)
            .then(function () {
            expect(resolvedSecondPromise).toBe(true);
            done();
        })
            .catch(done);
    });
});
