"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCustomConfigFile = exports.dynamicRequire = exports.existsSync = exports.formatFileSize = exports.writeFile = exports.getFileSize = exports.readFile = exports.readFileSync = void 0;
var tslib_1 = require("tslib");
var fs_1 = tslib_1.__importDefault(require("fs"));
var path_1 = tslib_1.__importDefault(require("path"));
var readFileSync = function (log, filePath) {
    try {
        return fs_1.default.readFileSync(filePath, 'utf8');
    }
    catch (err) {
        log.error({ err: err, filePath: filePath }, 'Error reading file');
        throw err;
    }
};
exports.readFileSync = readFileSync;
var readFile = function (log, filePath) {
    return new Promise(function (resolve, reject) {
        fs_1.default.readFile(filePath, 'utf8', function (err, data) {
            if (err) {
                log.error({ err: err, filePath: filePath }, 'Error reading file');
                reject(err);
            }
            else {
                resolve(data);
            }
        });
    });
};
exports.readFile = readFile;
var getFileSize = function (log, filePath) {
    return new Promise(function (resolve, reject) {
        fs_1.default.stat(filePath, function (err, stats) {
            if (err) {
                log.error({ err: err, filePath: filePath }, 'Error getting file size');
                reject(err);
            }
            else {
                resolve(stats.size);
            }
        });
    });
};
exports.getFileSize = getFileSize;
var writeFile = function (log, filePath, fileContent) {
    return new Promise(function (resolve, reject) {
        fs_1.default.writeFile(filePath, fileContent, function (err) {
            if (err) {
                log.error({ err: err, filePath: filePath }, 'Error writing file');
                reject(err);
            }
            else {
                resolve({});
            }
        });
    });
};
exports.writeFile = writeFile;
var formatFileSize = function (size) { return (size / 1024).toFixed(4); };
exports.formatFileSize = formatFileSize;
var existsSync = function (file) {
    try {
        fs_1.default.accessSync(file);
        return true;
    }
    catch (ex) {
        return false;
    }
};
exports.existsSync = existsSync;
function dynamicRequire(file) {
    if (typeof __non_webpack_require__ !== 'undefined') {
        return __non_webpack_require__(file);
    }
    else {
        return eval('module.require')(file);
    }
}
exports.dynamicRequire = dynamicRequire;
function getCustomConfigFile(log, root, filePath, fallback) {
    var customConfigFile = path_1.default.resolve(root, filePath + '.js');
    var customConfigFileTS = path_1.default.resolve(root, filePath + '.ts');
    if ((0, exports.existsSync)(customConfigFile)) {
        var config = dynamicRequire(customConfigFile).default;
        if (typeof config === 'function') {
            return config(root);
        }
        return config;
    }
    if ((0, exports.existsSync)(customConfigFileTS)) {
        log.error(filePath + ' only found as TypeScript file.');
        log.error('Please make sure to compile all TS files in the /config folder to JS!');
        process.exit(1);
    }
    return fallback;
}
exports.getCustomConfigFile = getCustomConfigFile;
