"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PromiseTracker = void 0;
var tslib_1 = require("tslib");
var PromiseTracker = (function () {
    function PromiseTracker() {
        this.waitIndex = 1;
        this.promises = {};
    }
    PromiseTracker.prototype.track = function (promise) {
        var currentWaitIndex = this.waitIndex;
        if (!this.promises[currentWaitIndex]) {
            this.promises[currentWaitIndex] = [];
        }
        if (promise.catch) {
            ;
            promise.catch(function () { });
        }
        this.promises[currentWaitIndex].push(promise);
    };
    PromiseTracker.prototype.middleware = function () {
        var _this = this;
        return function () { return function (next) { return function (action) {
            var result = next(action);
            if (Promise.resolve(result) === result) {
                _this.track(result);
            }
            return result;
        }; }; };
    };
    PromiseTracker.prototype.hasWork = function () {
        if (!this.promises[this.waitIndex]) {
            return false;
        }
        return this.promises[this.waitIndex].length > 0;
    };
    PromiseTracker.prototype.waitForCompletion = function () {
        if (!this.hasWork()) {
            return Promise.resolve();
        }
        var all = Promise.all(tslib_1.__spreadArray([], this.promises[this.waitIndex++], true))
            .then(function () { return new Promise(function (resolve) { return setTimeout(resolve, 0); }); });
        return all;
    };
    return PromiseTracker;
}());
exports.PromiseTracker = PromiseTracker;
