import { __spreadArray } from "tslib";
import fs from 'fs';
import path from 'path';
var assets;
var assetsLoaded = false;
export function getAssetsFile(base) {
    return path.resolve(base, 'assets.json');
}
function ensureAssets(runtimeConfig) {
    if (assetsLoaded) {
        return;
    }
    var assetsFile = getAssetsFile(runtimeConfig.BASE);
    try {
        var assetsFileContents = fs.readFileSync(assetsFile);
        assets = JSON.parse(assetsFileContents.toString());
        assetsLoaded = true;
    }
    catch (e) {
        throw new Error("Cannot read assets file from ".concat(assetsFile));
    }
}
export function setDefaultAssets(buildConfig) {
    updateAssetLocations({
        main: {
            css: buildConfig.PUBLIC_PATH + buildConfig.ASSETS_PATH_PREFIX + 'css/main.css',
            js: buildConfig.PUBLIC_PATH + buildConfig.ASSETS_PATH_PREFIX + 'js/main.js',
        },
    });
}
export function updateAssetLocations(newAssets) {
    assets = newAssets;
    assetsLoaded = true;
}
export function getAssetLocations(runtimeConfig) {
    ensureAssets(runtimeConfig);
    return assets;
}
export function getAssets(runtimeConfig) {
    ensureAssets(runtimeConfig);
    return assets;
}
export function getAbsoluteAssetPath(runtimeConfig, asset) {
    var staticPath = runtimeConfig.PUBLIC_PATH + runtimeConfig.ASSETS_PATH_PREFIX;
    var relativeAsset = asset.slice(staticPath.length);
    if (relativeAsset[0] === '/') {
        relativeAsset = relativeAsset.slice(1);
    }
    return path.resolve(runtimeConfig.ASSETS_PATH, relativeAsset);
}
export function getHeadAssets(buildAssets) {
    return Object.keys(buildAssets)
        .filter(function (chunkName) {
        var chunk = buildAssets[chunkName];
        return chunk.css;
    })
        .map(function (chunkName) { return ({
        tag: "<link id=\"css-main\" type=\"text/css\" rel=\"stylesheet\" href=\"".concat(buildAssets[chunkName].css, "\">"),
    }); });
}
export function getBodyAssets(buildAssets) {
    var chunkNames = Object.keys(buildAssets).filter(function (chunkName) {
        if (chunkName === 'main' || chunkName === 'manifest') {
            return false;
        }
        var chunk = buildAssets[chunkName];
        return !!chunk.js;
    });
    return __spreadArray(__spreadArray(['manifest'], chunkNames, true), ['main'], false).filter(function (chunkName) { return buildAssets[chunkName]; })
        .reduce(function (acc, chunkName) {
        var jsAsset = buildAssets[chunkName].js;
        if (Array.isArray(jsAsset)) {
            acc.push.apply(acc, jsAsset.map(function (file) { return ({
                tag: "<script src=\"".concat(file, "\"></script>"),
            }); }));
        }
        else {
            acc.push({
                tag: "<script src=\"".concat(buildAssets[chunkName].js, "\"></script>"),
            });
        }
        return acc;
    }, []);
}
