import express from 'express';
import { setDefaultAssets } from './assets';
import { getDefaultHtmlMiddleware } from './middleware/default-html-middleware';
import { createEnsureRequestLogMiddleware } from './middleware/ensure-request-log-middleware';
import { getRuntimeConfig, getBuildConfig } from './config/config';
import { isWatchtowerServer, projectDir, isWatchMode } from './environment';
import { findFreePort } from './utils/network';
var isProduction = process.env.NODE_ENV === 'production';
export var getPort = function (fallbackPort) {
    var port = parseInt(process.env.PORT || '', 10) || fallbackPort;
    if (!port) {
        throw new Error('No port specified, set one through the PORT environmental variable');
    }
    return port;
};
export function createServer(options) {
    var config = getRuntimeConfig(options.log);
    var earlyMiddlewareHook = options.earlyMiddlewareHook, middlewareHook = options.middlewareHook, callback = options.callback, _a = options.startListening, startListening = _a === void 0 ? true : _a;
    var app = express();
    app.log = options.log;
    app.disable('x-powered-by');
    if (isWatchtowerServer()) {
        var buildConfig = getBuildConfig(options.log, projectDir() || process.cwd());
        setDefaultAssets(buildConfig);
    }
    if (process.env.NODE_ENV !== 'production' && isWatchMode()) {
        var buildConfig = getBuildConfig(options.log, projectDir() || process.cwd());
        setDefaultAssets(buildConfig);
        var getHotReloadMiddleware = require('@project-watchtower/cli').getHotReloadMiddleware;
        app.use(getHotReloadMiddleware(options.log, buildConfig));
    }
    app.use(createEnsureRequestLogMiddleware(options.log));
    if (earlyMiddlewareHook) {
        earlyMiddlewareHook(app);
    }
    var assetsPathPrefixWithLeadingSlash = config.ASSETS_PATH_PREFIX[0] === '/'
        ? config.ASSETS_PATH_PREFIX
        : "/".concat(config.ASSETS_PATH_PREFIX);
    app.use(assetsPathPrefixWithLeadingSlash, express.static(config.ASSETS_PATH, {
        index: false,
    }));
    if (config.SERVER_PUBLIC_DIR) {
        app.use(express.static(config.SERVER_PUBLIC_DIR, {
            index: false,
        }));
    }
    if (middlewareHook) {
        middlewareHook(app);
    }
    app.get('*', getDefaultHtmlMiddleware(options.log, config));
    if (!startListening) {
        return app;
    }
    var listen = function (usePort) {
        var server = app.listen(usePort, function () {
            options.log.info("Server listening on port ".concat(usePort));
            if (process.env.NODE_ENV !== 'production' && isWatchMode()) {
                var openBrowser = require('@project-watchtower/cli').openBrowser;
                openBrowser(options.log, usePort);
            }
            if (callback) {
                callback();
            }
        });
        app.set('server', server);
    };
    var port = getPort();
    if (isProduction) {
        listen(port);
    }
    else {
        findFreePort(port).then(function (usePort) { return listen(usePort); });
    }
    return app;
}
