/// <reference types="react" />
import { Assets } from 'assets-webpack-plugin';
import { Express, Request, RequestHandler } from 'express';
import { HelmetData } from 'react-helmet';
import { Logger } from 'typescript-log';
import { PageTags } from './';
import { PageTag } from './full-render';
import { PromiseTracker } from '../utils/promise-tracker';
export interface RenderContext<SSRRequestProps = object> {
    ssrRequestProps: SSRRequestProps;
    promiseTracker: PromiseTracker;
}
export type RenderApp<SSRRequestProps extends object> = (params: {
    log: Logger;
    context: RenderContext<SSRRequestProps>;
    req: Request;
}) => JSX.Element;
export interface RenderHtmlParams<SSRRequestProps extends object, RenderResult> {
    head: HelmetData | undefined;
    renderResult: RenderResult;
    pageTags: PageTags;
    context: RenderContext<SSRRequestProps>;
    req: Request;
}
export type RenderHtml<SSRRequestProps extends object, RenderResult> = (params: RenderHtmlParams<SSRRequestProps, RenderResult>) => string;
export type CreatePageTags<SSRRequestProps> = (options: {
    buildAssets: Assets;
    helmetTags: string[];
    stateTransfers: PageTag[];
    renderContext: RenderContext<SSRRequestProps>;
}) => PageTags;
export interface ServerSideRenderMiddlewareOptions<SSRRequestProps extends object, RenderResult> {
    app: Express & {
        log: Logger;
    };
    ssrTimeoutMs: number;
    setupRequest: (req: Request, promiseTracker: PromiseTracker) => Promise<SSRRequestProps>;
    renderApp: RenderApp<SSRRequestProps>;
    renderHtml: RenderHtml<SSRRequestProps, RenderResult>;
    errorLocation: string;
    createPageTags?: CreatePageTags<SSRRequestProps>;
    pageNotFoundLocation: string;
    renderFn?: (element: React.ReactElement<any>) => RenderResult;
}
export declare const createSsrMiddleware: <SSRRequestProps extends object, RenderResult = string>(options: ServerSideRenderMiddlewareOptions<SSRRequestProps, RenderResult>) => RequestHandler;
