import { __awaiter, __generator } from "tslib";
var delay = function (forMs) { return new Promise(function (resolve) { return setTimeout(resolve, forMs); }); };
var component = 'RecursiveTaskResolver';
var timedOut = Symbol('Timed out');
function innerResolve(log, promiseTracker, render, location, initialRender, numberAttempts, remainingAttempts, timeoutPromise) {
    if (!promiseTracker.hasWork()) {
        log.debug({ component: component, location: location }, 'No work, resolving');
        return Promise.resolve(initialRender);
    }
    if (!remainingAttempts || remainingAttempts <= 0) {
        return Promise.reject(new Error("Abort waiting for loading all data after ".concat(numberAttempts, " recursive waits")));
    }
    return new Promise(function (resolve, reject) {
        return setTimeout(function () {
            var promiseOrTimedOut = [promiseTracker.waitForCompletion(), timeoutPromise];
            Promise.race(promiseOrTimedOut)
                .then(function (p) {
                if (p === timedOut) {
                    return p;
                }
                log.debug({ component: component, location: location }, 'Re-rendering to trigger any child promises');
                var renderResult = render();
                return innerResolve(log, promiseTracker, render, location, renderResult, numberAttempts, remainingAttempts - 1, timeoutPromise);
            })
                .then(resolve)
                .catch(reject);
        });
    });
}
export function recursiveTaskResolver(log, promiseTracker, render, location, initialRender, numberAttempts, timeoutMs) {
    return __awaiter(this, void 0, void 0, function () {
        var timeoutPromise, resolved;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    timeoutPromise = delay(timeoutMs).then(function () { return timedOut; });
                    return [4, innerResolve(log, promiseTracker, render, location, initialRender, numberAttempts, numberAttempts, timeoutPromise)];
                case 1:
                    resolved = _a.sent();
                    if (resolved === timedOut) {
                        log.error({ location: location }, "Resolving tasks timed out after ".concat(timeoutMs, "ms"));
                        return [2, render()];
                    }
                    return [2, resolved];
            }
        });
    });
}
