import fs from 'fs';
import path from 'path';
export var readFileSync = function (log, filePath) {
    try {
        return fs.readFileSync(filePath, 'utf8');
    }
    catch (err) {
        log.error({ err: err, filePath: filePath }, 'Error reading file');
        throw err;
    }
};
export var readFile = function (log, filePath) {
    return new Promise(function (resolve, reject) {
        fs.readFile(filePath, 'utf8', function (err, data) {
            if (err) {
                log.error({ err: err, filePath: filePath }, 'Error reading file');
                reject(err);
            }
            else {
                resolve(data);
            }
        });
    });
};
export var getFileSize = function (log, filePath) {
    return new Promise(function (resolve, reject) {
        fs.stat(filePath, function (err, stats) {
            if (err) {
                log.error({ err: err, filePath: filePath }, 'Error getting file size');
                reject(err);
            }
            else {
                resolve(stats.size);
            }
        });
    });
};
export var writeFile = function (log, filePath, fileContent) {
    return new Promise(function (resolve, reject) {
        fs.writeFile(filePath, fileContent, function (err) {
            if (err) {
                log.error({ err: err, filePath: filePath }, 'Error writing file');
                reject(err);
            }
            else {
                resolve({});
            }
        });
    });
};
export var formatFileSize = function (size) { return (size / 1024).toFixed(4); };
export var existsSync = function (file) {
    try {
        fs.accessSync(file);
        return true;
    }
    catch (ex) {
        return false;
    }
};
export function dynamicRequire(file) {
    if (typeof __non_webpack_require__ !== 'undefined') {
        return __non_webpack_require__(file);
    }
    else {
        return eval('module.require')(file);
    }
}
export function getCustomConfigFile(log, root, filePath, fallback) {
    var customConfigFile = path.resolve(root, filePath + '.js');
    var customConfigFileTS = path.resolve(root, filePath + '.ts');
    if (existsSync(customConfigFile)) {
        var config = dynamicRequire(customConfigFile).default;
        if (typeof config === 'function') {
            return config(root);
        }
        return config;
    }
    if (existsSync(customConfigFileTS)) {
        log.error(filePath + ' only found as TypeScript file.');
        log.error('Please make sure to compile all TS files in the /config folder to JS!');
        process.exit(1);
    }
    return fallback;
}
